/* -*- mode: c++; c-basic-offset: 3; -*- */
#ifndef WPIPE_WOUTPUT_HH
#define WPIPE_WOUTPUT_HH
#include "wtypes.hh"

#include <map>

namespace wpipe {

   /**  The %wouttype class holds a list of channels names and a file path 
     *  for each channel. The file paths are generated for each channel
     *  named in the constructor.
     */
   class wouttype {
   public:
      wouttype(void);
      wouttype(const str_vect& channels, const std::string& time, 
	       const std::string& dir, const std::string& fmt, 
	       const std::string& type);
      ~wouttype(void);
      void addChannels(const str_vect& channels, const std::string& time);
      size_t find(const std::string& chan) const;
      const std::string& path(const std::string& chan) const;
      const std::string& path(size_t i) const;
      size_t size(void) const;
      void update(const std::string& time);
   private:
      str_vect channelNames;
      str_vect _paths;
      std::string out_dir;
      std::string format;
      std::string trig_type;
   };

   /**  Maintain a list of output file names defined for multiple output 
     *  files defined for this analysis.
     */
   class woutput {
   public:
      woutput(void);
      woutput(const str_vect& channels, const std::string& time, 
	      const std::string& dir, const std::string& fmt, 
	      const str_vect& types);
      ~woutput(void);
      void addMany(const str_vect& channels, const std::string& time, 
		   const std::string& dir, const std::string& fmt, 
		   const str_vect& types);
      void addtype(const str_vect& channels, const std::string& time,
		   const std::string& dir, const std::string& fmt, 
		   const std::string& type);
      bool enabled(const std::string& name) const;
      const wouttype& operator[](const std::string& name) const;
      wouttype& operator[](const std::string& name);
      void update(const std::string& time, const str_vect& types);
   private:
      typedef std::map<std::string, wouttype> output_map;
      typedef output_map::value_type omap_value;
      typedef output_map::iterator   omap_iter;
      output_map out_map;
   };

   //======================================  Inline accessors
   inline const std::string&
   wouttype::path(size_t i) const {
      return _paths[i];
   }

   inline const std::string&
   wouttype::path(const std::string& chan) const {
      static std::string null;
      size_t i = find(chan);
      if (i == size()) return null;
      return _paths[i];
   }

   inline size_t
   wouttype::size(void) const {
      return _paths.size();
   }

   inline bool
   woutput::enabled(const std::string& name) const {
      return out_map.find(name) != out_map.end();
   }

   inline const wouttype&
   woutput::operator[](const std::string& name) const {
      return out_map.find(name)->second;
   }

   inline wouttype&
   woutput::operator[](const std::string& name) {
      return out_map.find(name)->second;
   }

}  // namespace wpipe

#endif // !defined(WPIPE_WOUTPUT_HH)
